
rm(list=ls())


library(stats)
library(nleqslv)
library(KernSmooth)
library(splines)
library(survival)
library(numDeriv)
library(statmod)
library(Rcpp)
library(RcppArmadillo)

# ------------------------ load working data: CPP with 1000 subjects
labor <- read.csv('workingdata_cpp1000.csv')
pid <- labor$pregid
nn <- length(unique(pid))

nid<-!duplicated(pid) 
did<-c(nid[-1],TRUE)
sid <- which(nid)  # location of first examination for each subject
eid <- which(did)  # location of last examination for each subject

dilation<-labor$dilation   # number of events up to each examination time

obs_time <- labor$obsTime   # examination time calculated from admission

ttalln=malln=list()
Kn<-rep(0,nn)
tyn=mendn=rep(0,nn)

x1<-labor$MomAge[nid] 

x1n<-x1/40      # rescaled momage for analysis
x2n<-labor$underweight[nid]  # underweight indicator
x3n<-labor$overweight[nid]  # overweight indicator
x4n<- labor$obese[nid]   # obese indicator

xn <- cbind(x1n,x2n,x3n,x4n)


for(i in 1:nn){
   tempt<-obs_time[sid[i]:eid[i]]
   tempm<-dilation[sid[i]:eid[i]]
   Kn[i]<-length(tempt)   # total number of examinations
   ttalln[[i]]<-round(100*tempt)/100    # examination times calculated from admission
   malln[[i]]<-tempm-c(0,tempm[-Kn[i]])   # number of events occurred between consecutive examinations
   tyn[i]<-round(100*ttalln[[i]][Kn[i]])/100   # last examination time calculated from admission
   mendn[i]<-sum(malln[[i]])   # total number of events observed by the last examination time
}

max(tyn) # maximum examination time is 23.52 hours


# ------------------------------- Analysis 

# the negative log likelihood functions are written in C++ to improve speed
sourceCpp("./Rcode_gamma_exp2_Qplus1_bmicat.cpp")

tau1<-4
tau2<-24        # upper bound of observation period
tau<-tau1+tau2  # upper bound of total duration of labor
Q<-499  		# number of Riemann sum partitions

od<-3		# order of I-splie
kn<-12		# number of interior knots of I-spline
p<-kn+od	# number of I-spline base functions

pn <- dim(xn)[2]

tseq<-seq(0,tau,0.01)
size_tseq<-length(tseq)

obs<-unlist(ttalln)
interknot<-quantile(obs[obs!=0],probs=seq(1,kn)/kn)
knot<-c(rep(0,od),interknot,rep(tau,od))


## generate I-spline matrix
ispmt<-matrix(0,p,length(tseq))
for(i in 1:p){
  for(j in 1:length(tseq)){
    temp<-splineDesign(knot,tseq[j],ord=od,outer.ok=TRUE)
    ispmt[i,j]<-sum(temp[i:p])
  }
}


## The R-version of the likelihood functions are provided below, but the C++ function negPnc() is used for computational efficiency.

L1<-function(lxi,malli,Ki,indti){
    xi<-c(0,exp(lxi))/sum(exp(lxi))
   indti<-pmin(indti,length(tseq))
   temp<-((xi%*%(ispmt[,indti[2:(Ki+1)]]-ispmt[,indti[1:Ki]])+1e-300)/(xi%*%ispmt[,indti[Ki+1]]%*%rep(1,Ki)+1e-300))^(malli[1:Ki])
   return(temp+1e-300)
}


# average
negPn<-function(kexi){
  kap<-kexi[1]
  eta<-kexi[2:(pn+1)]
  lxi<-kexi[-(1:(pn+1))]
  suml<-0
  for(i in 1:nn){
     Exi<-0
     # The following line assumes that admission time follows a Gamma distribution. Change accordingly if using other distributions.
     Atemp<-qgamma(seq(1/(Q+1),Q/(Q+1),1/(Q+1)),shape=exp(kap),scale=exp(xn[i,]%*%eta))
     for(q in 1:Q){
        tallq<-ttalln[[i]]+round(100*Atemp[q])/100
        tq0<-c(0,tallq)
        indti<-round(tq0*100+1)
        Ltemp1<-L1(lxi,malln[[i]],Kn[i],indti)
        Exi<-Exi+prod(Ltemp1)
     }
     iadj<-0
     while(Exi/Q==0 & iadj<20){
         Exi<-0
         iadj<-iadj+1
         for(q in 1:Q){
            tallq<-ttalln[[i]]+round(100*Atemp[q])/100
            tq0<-c(0,tallq)
            indti<-round(tq0*100+1)
            Ltemp1<-L1(lxi,malln[[i]],Kn[i],indti)*10^iadj
            Exi<-Exi+prod(Ltemp1)
         }        
     }
     suml<-suml-log(Exi/(Q+1))+Kn[i]*iadj*log(10)
#     print(c(i,suml))
  }
  return(suml)
}
#---------------------------------------------------------------



#******* The optimization could be slightly sensitive to initial value, one could try several different initials and pick the one that gives best likelihood value. 

xi1<-rep(1/(p-1),p-1)
lxi1<-log(xi1)
keta1<-c(0,rep(0.1,pn))
kexi0<-c(keta1,lxi1)


#******** There is also slight sensitivity of the optimization result accross platforms (Windows, Mac, etc.) 
theta2<-optim(kexi0,negPnc,method="L-BFGS-B",lower=rep(-5,p+pn),upper=c(rep(3,pn+1),rep(0,p-1)),control=list(maxit=1e3,lmm=20))

theta2$value  # the negative log likelihood value at optimum parameters


## estimates of admission time parameters and F_n
kexih<-theta2$par
kaph<-kexih[1]
etah<-kexih[2:(pn+1)] # 
lxih<-kexih[-(1:(pn+1))]
xih<-c(0,exp(lxih)/sum(exp(lxih)))

Fn<-xih%*%ispmt



# use age 24, normal weight for the estimates 
xm <- c(0.6,0,0,0)

## Plot the distribution of Time to Admission (Gamma distribution assumed, change accordingly if other distribution is used.)
fA<-function(t){
   dgamma(t,shape=exp(kaph),scale=exp(xm%*%etah))
}

SA<-function(t){
   1-pgamma(t,shape=exp(kaph),scale=exp(xm%*%etah))
}  
upA<-qgamma(0.999,shape=exp(kaph),scale=exp(xm%*%etah))
Aseq<-seq(0,upA,0.01)

plot(Aseq,fA(Aseq),type="l",xlab='Time (hour)',ylab='Density',main='Density curve of time to admission',
     xaxt="n",yaxt="n",bty="n",font.lab=2,cex.lab=1.5,cex.main=1.5,lwd=2)
box(lwd=2)
axis(1,seq(0,30,5),lwd=2,font.axis=2,cex.axis=1.3)
axis(2,seq(0,0.12,0.02),lwd=2,font.axis=2,cex.axis=1.3)


## estimating equations (8), for estimation of beta and Lambda_0(tau)
xstarn<-cbind(1,xn)
esteq1<-function(bstar){
   right<-rep(0,nn)
   for(i in 1:nn){
   	# Gamma distribution assumed here. Change accordingly if other distribution is used.
      Ah<-qgamma(seq(1/(Q+1),Q/(Q+1),1/(Q+1)),shape=exp(kaph),scale=exp(xn[i,]%*%etah))
      tempi<-0
      yq<-tyn[i]+round(100*Ah)/100
      indyq<-yq*100+1
      indyq<-pmin(indyq,length(tseq))
      tempq<-sum(Fn[indyq])/(Q+1)
      if(tempq==0){
         right[i]<-0
      }else{
         right[i]<-mendn[i]/tempq-exp(xstarn[i,]%*%bstar)
      }
   }
   result<-t(xstarn)%*%right
   return(result)
}


bstar<-nleqslv(c(1,rep(0,pn)),esteq1)$x
Lamntau<-exp(bstar[1])
bhat<-bstar[-1]

Lamest<-Fn*Lamntau

Lamn<-function(t){
  id<-sum(tseq<=t)
  Lamest[id]
}



## estimating equation for gamma and Lambda_{H0}(tau2)
esteq2<-function(gstar){
   result<-t(xstarn)%*%(Kn-1-exp(xstarn%*%gstar))
   return(result)
}

gstar<-nleqslv(c(1,rep(0,pn)),esteq2)$x
ghat<-gstar[-1]


## estimate latent variable using equation (9)
zhat<-as.vector((Kn-1)/exp(xstarn%*%gstar))

## estimate distribution of latent variable using kernel smoothing
h<-dpik(zhat)
ksmg <- bkde(zhat, bandwidth=h,range.x=range(zhat),truncate=FALSE)

## by default, it gives the estimated density values at 401 equally spaced points from min(zhat) to max(zhat)
zz<-ksmg$x;zd<-ksmg$y
stepz<-(max(zz)-min(zz))/400
ptrue<-sum(zd[1:400]*stepz)
zdt<-ksmg$y/ptrue


## survival function of the first gap time
S1<-function(t){
  sum(exp(-zz[1:400]*Lamn(t)*exp(xm%*%bhat))*zdt[1:400]*stepz)
}

Median1<-function(t){
  S1(t)-1/2
}

quant1<-function(t,p){
  S1(t)-1+p
}

ww<-rep(0,10)
tt<-rep(0,10)

ww[1]<-uniroot(Median1,c(0,20))$root
tt[1]<-ww[1]

## survival function of the (k+1)-th gap time given the T_k=tk
Skk<-function(t,tk){
  sum(exp(-zz[1:400]*(Lamn(t+tk)-Lamn(tk))*exp(xm%*%bhat))*zdt[1:400]*stepz)
}

Mediankk<-function(t,tk){
  Skk(t,tk)-1/2
}


quantkk<-function(t,tk,p){
  Skk(t,tk)-1+p
}


for(k in 2:10){
  ww[k]<-uniroot(Mediankk,tk=tt[k-1],c(0,20))$root
  tt[k]<-tt[k-1]+ww[k]
}


# twseq: matrix of the following 11 quantiles for the 10 gap times
pp<-c(0.05,0.1,0.2,0.25,0.4,0.5,0.6,0.75,0.8,0.9,0.95)
twseq<-matrix(0,10,11)
indw<-matrix(0,10,11)

for(j in 1:11){
twseq[1,j]<-uniroot(quant1,p=pp[j],c(0,20))$root
}

for(k in 2:10){
   for(j in 1:11){
      twseq[k,j]<-uniroot(quantkk,tk=tt[k-1],p=pp[j],c(0,20))$root
   }
}


tts <- seq(0,10,0.01)


## matrix of estimated survival functions of the 1st to 10th gap times, each evaluated at tts
S1to10h <- matrix(0,10,length(tts))
S1to10h[1,] <- sapply(tts,S1) 
for(k in 2:10){
	S1to10h[k,] <- sapply(tts,Skk,tk=tt[k-1])
}
