#### R code sample for Scenario II with unknown start time, assuming Gamma distributed admission time
rm(list=ls())

library(stats)
library(nleqslv)
library(KernSmooth)
library(splines)
library(survival)
library(numDeriv)
library(statmod)

n <- 250
tau1<-7.0
tau2<-8.0       # upper bound of observation period
tau<-tau1+tau2  # upper bound of total duration of labor

lambdas0<-1.0  # true baseline intensity function of observation process 
beta0<-1.5     # true regression coefficient of event process
gamma0<-1.0    # true regression coefficient of observation process

kap0<- 0.5     # exp(kap0) is true shape parameter of admission time
eta0<- -1.0    # exp(x*eta0) is true scale parameter of admission time given covariate x


cc<-30
# true F(t) function = Lambda_0(t)/Lambda_0(tau)
Ftrue<-function(t){
  (t^2+2*cc*t)/(tau^2+2*cc*tau)
}

# true Lambda_0(t) function
Lamtrue<-function(t){
  (t^2+2*cc*t)/(tau+2*cc)
}

od <- 3  # order of I-spline
kn <- 4  # number of interior knots of I-splines
p <- kn + od  # number of I-spline base functions


## All time points are rounded to 2 decimal places
tseq = seq(0, tau, 0.01)

## select inter knots as 20%, 40%, 60% and 80% quantiles of all examination times
knot<-c(rep(0,od),c(2.08,3.81,5.53,7.24),rep(tau,od))   

## generate I-spline matrix
ispmt<-matrix(0,p,length(tseq))
for(i in 1:p){
  for(j in 1:length(tseq)){
    temp<-splineDesign(knot,tseq[j],ord=od,outer.ok=TRUE)
    ispmt[i,j]<-sum(temp[i:p])
  }
}



# -------------------- generate data --------------------------

z<-rgamma(n,shape=2,scale=1/2)  # latent variable z
x<-runif(n,0,1)   # covariate x
pn <- 1 # dimension of x

J<-rpois(n,z*tau*exp(x*beta0))  # number of events occurred by time tau

# T: true event times
# tall: examination times calculated from true time zero
# ttall: examination times calculated from admission time
# K: number of examinations including the one at admission
# y: last examination time calculated from true time zero
# ty: last examination time calculated from admission time

T=tall=ttall=mall=list()
K=y=ty=mend=rep(0,n)

# Gamma distribution assumed for admission time. Change accordingly if other distribution is used.
Ai<-round(100*rgamma(n,shape=exp(kap0),scale=exp(x*eta0)))/100


for(i in 1:n){
    if(Ai[i]<=tau){
       tempkk<-rpois(1,z[i]*lambdas0*tau2*exp(x[i]*gamma0))
       K[i]<-tempkk+1
       if(tempkk>0){
          temptt<-round(100*runif(tempkk,Ai[i],Ai[i]+tau2))/100
          tall[[i]]<-c(Ai[i],sort(temptt)) # true examination times which are unknown
       }else{
          tall[[i]]<-Ai[i]
       }
       ttall[[i]]<-tall[[i]]-Ai[i]
       if(J[i]>0){
          u<-runif(J[i],0,1)
          tempT<-sqrt(((tau)^2+2*cc*(tau))*u+cc^2)-cc
          T[[i]]<-sort(tempT)
       }else{
          T[[i]]<-tau+1
       }
       mall[[i]]<-rep(0,K[i])
       for(j in 1:K[i]){
          if(j==1){
             mall[[i]][j]<-sum(T[[i]]<=tall[[i]][j])
          }else{
             mall[[i]][j]<-sum(T[[i]]>tall[[i]][j-1] & T[[i]]<=tall[[i]][j])
          }
       }
       y[i]<-tall[[i]][K[i]]
       mend[i]<-sum(T[[i]]<=y[i])
       ty[i]<-y[i]-Ai[i]
    }
}


# remove subjects with no observations (very very rare)
id<-(K>0)
nn<-sum(id)
Kn<-K[id]
xn<-x[id]
ttalln<-ttall[id]
malln<-mall[id]
tyn<-ty[id]
mendn<-mend[id]


## conditional likelihood contribution of subject i given admission time 
L1 <- function(lxi,malli,Ki,indti){
	#Ki: K_i, number of observations for subject i
	#malli: vector (m_{i1},...,m_{iK_i})
	#indti: vector of indicators of location for time (0,t_{i1},...,t_{iK_i}) in the tseq
	xi <- c(0,exp(lxi))/sum(exp(lxi)) # vector of I-spline coefficients for F_n(t)
	indti <- pmin(indti, length(tseq)) # F_n(t) remains constant when t >= tau
	temp<-((xi%*%(ispmt[,indti[2:(Ki+1)]]-ispmt[,indti[1:Ki]])+1e-300)/(xi%*%ispmt[,indti[Ki+1]]%*%rep(1,Ki)+1e-300))^(malli[1:Ki])
   return(temp+1e-300)
}


## full log likelihood function using Riemann sum approximation 
# Q+1 : number of Riemann sum partitions
Q <- 199

negPn<-function(kexi){
  kap<-kexi[1] # exp(kap) is shape parameter of admission time
  eta<-kexi[2:(pn+1)]  # exp(xn*eta) is scale parameter of admission time
  lxi<-kexi[-(1:(pn+1))]
  suml<-0
  for(i in 1:nn){
     Exi<-0
     # Gamma distribution for admission time assumed here. Change accordingly if other distribution is used.
     Atemp<-qgamma(seq(1/(Q+1),Q/(Q+1),1/(Q+1)),shape=exp(kap),scale=exp(xn[i]*eta)) # get quantiles of admission time for Riemann sum
     for(q in 1:Q){
        tallq<-ttalln[[i]]+round(100*Atemp[q])/100
        tq0<-c(0,tallq)
        indti<-round(tq0*100+1)
        Ltemp1<-L1(lxi,malln[[i]],Kn[i],indti)
        Exi<-Exi+prod(Ltemp1)
     }
     # sometimes prod(Ltemp1) are too small, not distinguishable from 0 in R, do the following could help
     iadj<-0
     while(Exi/(Q+1)==0 & iadj<20){
         Exi<-0
         iadj<-iadj+1
         for(q in 1:Q){
            tallq<-ttalln[[i]]+round(100*Atemp[q])/100
            tq0<-c(0,tallq)
            indti<-round(tq0*100+1)
            Ltemp1<-L1(lxi,malln[[i]],Kn[i],indti)*10^iadj
            Exi<-Exi+prod(Ltemp1)
         }        
     }
     suml<-suml-log(Exi/(Q+1))+Kn[i]*iadj*log(10)
#     print(c(i,suml))
  }
  return(suml)
}



## set initial value for optimization
xi0<-c(0.0118,0.0941,0.1647,0.2353,0.3059,0.1882)
lxi0<-log(xi0)
kexi0<-c(kap0,eta0,lxi0)


low=c(-1,-3,rep(-5,p-1))  # low: lower bound for optimization if using "L-BFGS-B" method
upp=c(2,1,rep(0,p-1))     # upp: upper bound for optimization if using "L-BFGS-B" method

kexih<-optim(kexi0,negPn,method="L-BFGS-B",lower=low,upper=upp,control=list(maxit=1000,lmm=20))$par  

## estimates of admission time parameters and F_n
kaph<-kexih[1]
etah<-kexih[2]
xitemp<-exp(kexih[-(1:2)])
xih<-c(0,xitemp/sum(xitemp))
Fn<-xih%*%ispmt


## estimating equations (8), for estimation of beta and Lambda_0(tau)

xstarn<-cbind(1,xn)
esteq1<-function(bstar){
   right<-rep(0,nn)
   for(i in 1:nn){
   	# Gamma distribution assumed for admission time here. Change accordingly if other distribution is used.
      Ah<-qgamma(seq(1/(Q+1),Q/(Q+1),1/(Q+1)),shape=exp(kaph),scale=exp(xn[i]*etah))
      tempi<-0
      yq<-tyn[i]+round(100*Ah)/100
      indyq<-yq*100+1
      indyq<-pmin(indyq,length(tseq))
      tempq<-sum(Fn[indyq])/(Q+1)
      if(tempq==0){
         right[i]<-0
      }else{
         right[i]<-mendn[i]/tempq-exp(xstarn[i,]%*%bstar)
      }
   }
   result<-t(xstarn)%*%right
   return(result)
}

bstar<-nleqslv(c(1,0),esteq1)$x
Lamntau<-exp(bstar[1])
bhat<-bstar[-1]

## estimate of Lambda_0(t)
Lamest<-Fn*Lamntau

Lamn<-function(t){
  id<-sum(tseq<=t)
  Lamest[id]
}

## estimating equation for gamma and Lambda_{H0}(tau2)
esteq2<-function(gstar){
   result<-t(xstarn)%*%(Kn-1-exp(xstarn%*%gstar))
   return(result)
}

gstar<-nleqslv(c(1,0),esteq2)$x
ghat<-gstar[-1]

## estimate latent variable using equation (9)
zhat<-as.vector((Kn-1)/exp(xstarn%*%gstar))

## estimate distribution of latent variable using kernel smoothing
h<-dpik(zhat)
ksmg <- bkde(zhat, bandwidth=h,range.x=range(zhat),truncate=FALSE)

## by default, it gives the estimated density values at 401 equally spaced points from min(zhat) to max(zhat)
zz<-ksmg$x;zd<-ksmg$y
stepz<-(max(zz)-min(zz))/400
ptrue<-sum(zd[1:400]*stepz)
zdt<-ksmg$y/ptrue


## estimate survival functions, use Riemann sum approximation for integral w.r.t latent variable z
xm <- 0.5 # covariate value used for computing survival functions 
S1<-function(t){
  sum(exp(-zz[1:400]*Lamn(t)*exp(xm*bhat))*zdt[1:400]*stepz)
}

# tk: median of the k-th gap time 
Skk<-function(t,tk){
  sum(exp(-zz[1:400]*(Lamn(t+tk)-Lamn(tk))*exp(xm*bhat))*zdt[1:400]*stepz)
}

# true T_k, k=1,..., 5
tt<-c(0.4852,0.9628,1.4332,1.8966,2.3534) 

tts <- seq(0,4,0.01)
S1to5 <- matrix(0,5,length(tts))
S1to5[1,] <- sapply(tts,S1)
for(k in 2:5){
	S1to5[k,]<-sapply(tts, Skk, tk=tt[k-1])
}

